#############################################################################
# stringreq.tcl Copyright 1998, 2004 r0lZ
# String requester.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

# stringReq 1.1 - String requester, by r0lZ, September 1998 - February 2004
#
# Usage: stringreq -window .path args...
#
# Returns the entered string, or "" if user cancelled the requester or the window
# were destroyed.  When used without -window argument, puts string and exit Wish.
#
# Arguments:
# -callback procname Procedure to be called to verify the validity of user
#				input.  The procedure is called with the current content
#				of the string every time the user modify it, and must
#				return true to accept or false to reject the string.
# -cancel str	Text for the Cancel button.
# -def str		Default string in entry.
# -font str		Font name (current system font is the default)
# -geometry str	Geometry descriptor string (in the form XxY+W+H), or
#				"s" to center the requester on screen (the default), or
#				"w" to put it in the topleft corner of the parent window, or
#				"c" to center the requester under the mouse cursor.
# -hidechar char Character used to hide what the user types in.
# -highlight	Flag: highlight text
# -justify opt	"left" (the default), "center" or "right"
# -ok str		Text for the OK button.
# -text str		Message to display in dialog.
# -title str	Title to display in dialog's decorative frame.
# -width nchars	Width of text box in characters or 0 for auto-resize.
#				Default: 32
# -window		Window path to use for dialog top-level.
#				If omitted, this is the root window that is used, and
#				destroyed at end!  Use this for DOS batch inputs.
# -cancelreturn Text to return if the user cancelled the requester (default: "").

# TODO:
#	- Catch all possible errors.
#	- Return an error code 1 when req is destroyed. (?)

namespace eval ::stringreq {
namespace export stringreq

####### main proc
proc stringreq { args } {

	variable ::stringreq::textvar ::stringreq::done

	### args
	set _window ""
	set _title ""
	set _text ""
	set _def ""
	set _callback ""
	set _width 32
	set _justify "left"
	set _hidechar ""
	set _ok "OK"
	set _cancel "Cancel"
	set _geometry "s"
	set _highlight ""
	set _font ""
	set _cancelreturn ""
	set _highlight 0

	set ind 0
	set max [llength $args]
	while {$ind < $max} {
		switch -exact -- [lindex $args $ind] {
		  "-window"		{incr ind;set _window	[lindex $args $ind];incr ind}
		  "-title"		{incr ind;set _title	[lindex $args $ind];incr ind}
		  "-text"		{incr ind;set _text		[lindex $args $ind];incr ind}
		  "-def"		{incr ind;set _def		[lindex $args $ind];incr ind}
		  "-callback"	{incr ind;set _callback	[lindex $args $ind];incr ind}
		  "-width"		{incr ind;set _width	[lindex $args $ind];incr ind}
		  "-justify"	{incr ind;set _justify	[lindex $args $ind];incr ind}
		  "-hidechar"	{incr ind;set _hidechar	[lindex $args $ind];incr ind}
		  "-ok"			{incr ind;set _ok		[lindex $args $ind];incr ind}
		  "-cancel"		{incr ind;set _cancel	[lindex $args $ind];incr ind}
		  "-geometry"	{incr ind;set _geometry	[lindex $args $ind];incr ind}
		  "-font"		{incr ind;set _font		[lindex $args $ind];incr ind}
		  "-cancelreturn" {incr ind;set _cancelreturn [lindex $args $ind];incr ind}
		  "-highlight"	{incr ind;set _highlight 1}
		  default	{show_warning -icon error -title "Tcl/Tk error" -parent . \
			  -message "stringReq proc:\nunknown option [lindex $args $ind]"; \
			  return ""}
		}
	}

	if {$_window == ""} {
		set w "."
		set wp ""
	} else {
		set w $_window
		set wp $_window
	}

	### init
	set ::stringreq::done -1
	set ::stringreq::textvar $_def

	### build window
	if {$w != "."} {
		catch {destroy $w}
		toplevel $w -class Dialog
	}
	wm withdraw $w
	wm title $w $_title
	wm iconname $w Dialog
	wm protocol $w WM_DELETE_WINDOW {set ::stringreq::done 0}
	bind $w <Escape> "set ::stringreq::done 0"
	if {$::tcl_platform(platform) == "windows"} {
		catch {wm attributes $w -toolwindow 1}
	}
	if {$w != "."} {
		wm transient $w .
	}

	bind $w <Return>  {set ::stringreq::done 1}
	bind $w <Escape>  {set ::stringreq::done 0}
	#bind $w <Destroy> {set ::stringreq::done 0}
	wm protocol $w WM_DELETE_WINDOW {set ::stringreq::done 0}

	### build widgets

	# row 1: label
	if {$_text != ""} {
		if {!$_highlight} {
			label $wp.label -takefocus 0 -text $_text -padx 1m
		} else {
			label $wp.label -takefocus 0 -text $_text -padx 1m -pady 2m \
				-foreground brown \
				-background white \
				-borderwidth 2 -relief groove
		}
		pack $wp.label -expand true -fill both -padx 2 -pady 4
	}

	# row 2: entry
	entry $wp.entry -textvariable ::stringreq::textvar -width $_width \
		-show $_hidechar -justify $_justify
	pack $wp.entry -fill x  -padx 4

	# row 3: buttons
	frame $wp.bf;
	  set width [string length $_ok]
	  if {[string length $_cancel] > $width} {set width [string length $_cancel]}
	  # col 1: resizable blank space
	  pack [frame  $wp.bf.pad1] -side left -fill both -expand true
	  # col 2: OK button
	  button $wp.bf.okbutton -text $_ok -width $width -command {set ::stringreq::done 1}
	  pack $wp.bf.okbutton -side left -fill both -expand true
	  # col 3: Cancel button
	  button $wp.bf.cancelbutton -text $_cancel -width $width -command {set ::stringreq::done 0}
	  pack $wp.bf.cancelbutton -side left -fill both -expand true -padx 4
	pack $wp.bf -fill both -pady 4

	### Cosmetics stuff: set selection and focus to string gadget,
	### change window position if requested, and deiconify window
	#wm resizable $w yes no
	if {$_font != ""} {
		if {$_text != ""} {
			$wp.label configure -font $_font
		}
		$wp.entry configure -font $_font
		$wp.bf.okbutton configure -font $_font
		$wp.bf.cancelbutton configure -font $_font
	}

	update
	if {$w != "."} {
		switch $_geometry {
			"" {
				break
			}
			"c" {
				set x [expr [winfo pointerx .]-([winfo reqwidth $w]/2)]
				set y [expr [winfo pointery .]-([winfo reqheight $w]/2)-20]
				if {$x < 5} {set x 5}
				if {$y < 5} {set y 5}
				wm geom $w +$x+$y
			}
			"w" {
				wm geom $w +[winfo rootx .]+[winfo rooty .]
			}
			"s" {
				set x [expr {[winfo screenwidth $w]/2 \
					- [winfo reqwidth $w]/2 \
					- [winfo vrootx [winfo parent $w]]}]
				set y [expr {[winfo screenheight $w]/2 \
					- [winfo reqheight $w]/2 \
					- [winfo vrooty [winfo parent $w]]}]
				if {$x < 5} {set x 5}
				if {$y < 5} {set y 5}
				wm geom $w +$x+$y
			}
			default {
				catch [wm geom $w $_geometry]
			}
		}
	} else {
		switch $_geometry {
			"" {
				break
			}
			"c" {
				set x [expr [winfo pointerx .]-([winfo reqwidth $w]/2)]
				set y [expr [winfo pointery .]-([winfo reqheight $w]/2)-20]
				if {$x < 5} {set x 5}
				if {$y < 5} {set y 5}
				wm geom $w +$x+$y
			}
			"w" {
				break
			}
			"s" {
				set x [expr {[winfo screenwidth $w]/2 \
					- [winfo reqwidth $w]/2 \
					- [winfo vrootx $w]}]
				set y [expr {[winfo screenheight $w]/2 \
					- [winfo reqheight $w]/2 \
					- [winfo vrooty $w]}]
				if {$x < 5} {set x 5}
				if {$y < 5} {set y 5}
				wm geom $w +$x+$y
			}
			default {
				catch [wm geom $w $_geometry]
			}
		}
	}
	focus $wp.entry
	focus $w
	$wp.entry selection range 0 [string length $::stringreq::textvar]
	$wp.entry icursor [string length $::stringreq::textvar]
	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [expr [winfo reqwidth $w]*4] [winfo reqheight $w]
	wm deiconify $w
	if {$w != "."} {
		wm transient $w .
		grab set $w
	}

	### establist a callback on the variable that holds the string
	### to be able to correct it when a mistake is typed
	if {$_callback != ""} {
		$wp.entry configure -validatecommand "$_callback %P" -validate key
	} else {
		$wp.entry configure -validatecommand { } -validate none
	}

	bind $wp.entry <Control-z> "set ::stringreq::textvar $_def"

	### wait user
	tkwait variable ::stringreq::done

	### cleanup
	if $::stringreq::done {
		set ret $::stringreq::textvar
	} else {
		set ret $_cancelreturn
	}
	unset ::stringreq::textvar
	if {$w != "."} {
		unset ::stringreq::done
		grab release $w
		destroy $w
	} else {
		puts $ret
		exit $::stringreq::done
	}
	return $ret
}
}
